#!/usr/bin/env python
# coding=utf-8

"""
elzzur solves a Ruzzle board, listing all the valid words with their scores.
"""

from __future__ import absolute_import
from __future__ import print_function

__author__ = "Alberto Pettarin"
__copyright__ = "Copyright 2016, Alberto Pettarin (www.albertopettarin.it)"
__license__ = "MIT"
__version__ = "0.0.1"
__email__ = "alberto@albertopettarin.it"
__status__ = "Production"

from elzzur.__main__ import main



